from dateutil.parser import parse


class Util:
    @staticmethod
    def get_jst_timestamp_from_yyyymmdd(yyyymmdd: str) -> int:
        date = parse(f"{yyyymmdd} 00:00:00+09:00", default=parse("00:00Z"))
        return int(date.timestamp() * 1000)



import json
import os
import sys

import click

from . import constants
from .auth import Auth
from .client import Client


class Mash(object):
    pass


@click.group(invoke_without_command=True)
@click.option("--debug/--no-debug", default=False, help="enable debug logging")
@click.option(
    "--config", "-c", default=constants.DEFAULT_CONFIG_PATH, help="config path"
)
@click.option(
    "--profile", "-p", default=constants.DEFAULT_CONFIG_PROFILE, help="config path"
)
@click.option(
    "--version/--no-version", "-v", default=False, help="show version. (default: False)"
)
@click.pass_context
def cli(ctx, debug, config, profile, version):
    ctx.obj = Mash()
    auth = Auth(
        config_path=config,
        config_profile=profile,
    )
    ctx.obj.api = Client(auth=auth)
    if version:
        print(constants.VERSION)
        sys.exit()

    if ctx.invoked_subcommand is None:
        click.echo(ctx.get_help())


@cli.command(help="get access token")
@click.pass_context
def token(ctx):
    token = ctx.obj.api.auth.get_access_token()
    print(token)


@cli.command(help="Get game-schedule and output json.")
@click.option("--filter", help="filter param")
@click.pass_context
def game_schedule(ctx, filter):
    game_schedule = ctx.obj.api.get_game_schedule(filter)
    print(json.dumps(game_schedule))


@cli.command(help="Get pitch data and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def pitch_data(ctx, filter, limit):
    pitch_data = ctx.obj.api.get_pitch_data(filter, limit)
    print(json.dumps(pitch_data))


@cli.command(help="Get movie-angle-upload and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def movie_angle_upload(ctx, filter, limit):
    result = ctx.obj.api.get_movie_angle_upload(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get movie-angle and output json.")
@click.option("--filter", help="filter param")
@click.pass_context
def movie_angle(ctx, filter):
    result = ctx.obj.api.get_movie_angle(filter)
    print(json.dumps(result))


@cli.command(help="download file by url.")
@click.option("--url", required=True, help="pitchbase file download url.")
@click.option("--download_dir", required=True, help="download directory.")
@click.pass_context
def download(ctx, url, download_dir):
    ctx.obj.api.download(f"{download_dir}/{os.path.basename(url)}", url)


@cli.command(help="download movie from movie-angle.")
@click.option("--filter", help="filter param")
@click.option("--download_dir", required=True, help="download directory.")
@click.pass_context
def download_movie_angle_upload_movie_file_path(ctx, filter, download_dir):
    ctx.obj.api.download_movie_angle_upload_movie_file_path(filter, download_dir)


@cli.command(help="download play-list-file from pitch-data.")
@click.option("--filter", help="filter param")
@click.option("--download_dir", required=True, help="download directory.")
@click.pass_context
def download_pitch_data_playlist_file_path(ctx, filter, download_dir):
    ctx.obj.api.download_pitch_data_playlist_file_path(filter, download_dir)


@cli.command(help="download ts files from m3u8.")
@click.option("--m3u8_url", required=True, help="m3u8 url on pitchbase.")
@click.option("--download_dir", required=True, help="download directory.")
@click.pass_context
def download_ts_from_m3u8(ctx, m3u8_url, download_dir):
    ctx.obj.api.download_ts_from_m3u8(m3u8_url, download_dir)


@cli.command(help="Get player-master and output json.")
@click.option("--filter", help="filter param")
@click.pass_context
def player_master(ctx, filter):
    result = ctx.obj.api.get_player_master(filter)
    print(json.dumps(result))


@cli.command(help="Get team-master and output json.")
@click.option("--filter", help="filter param")
@click.pass_context
def team_master(ctx, filter):
    result = ctx.obj.api.get_team_master(filter)
    print(json.dumps(result))


@cli.command(help="Get umpire-master and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def umpire_master(ctx, filter, limit):
    result = ctx.obj.api.get_umpire_master(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get stadium-master and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def stadium_master(ctx, filter, limit):
    result = ctx.obj.api.get_stadium_master(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get trackman_record and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def trackman_record(ctx, filter, limit):
    result = ctx.obj.api.get_trackman_record(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get charlyze_record and output json.")
@click.option("--resource-name", "-r", help="resource name")
@click.option("--filter", help="filter param")
@click.pass_context
def charlyze_record(ctx, resource_name, filter):
    result = ctx.obj.api.get_charlyze_record(resource_name, filter)
    print(json.dumps(result))


@cli.command(help="Get event_info and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def event_info(ctx, filter, limit):
    result = ctx.obj.api.get_event_info(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get batter_record and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def batter_record(ctx, filter, limit):
    result = ctx.obj.api.get_batter_record(filter, limit)
    print(json.dumps(result))


@cli.command(help="Get pitcher_record and output json.")
@click.option("--filter", help="filter param")
@click.option("--limit", type=int, help="limit param")
@click.pass_context
def pitcher_record(ctx, filter, limit):
    result = ctx.obj.api.get_pitcher_record(filter, limit)
    print(json.dumps(result))


def main():
    cli(obj={})



import logging
from logging import getLogger, INFO, DEBUG, Logger


def get_logger(debug: bool = False) -> Logger:
    logging.basicConfig(
        format="%(asctime)s %(levelname)s - %(message)s",
    )
    logger = getLogger(__name__)
    logger.setLevel(DEBUG if debug else INFO)
    return logger



import os
from pathlib import Path
from typing import Tuple

import m3u8

from .auth import Auth
from .logger import get_logger


class Client:
    def __init__(self, auth: Auth, debug: bool = False):
        self.auth = auth
        self.headers = self.auth.get_header()
        self.logger = get_logger(debug=debug)

    def get_request(self, path: str, params: dict) -> dict:
        rs = self.auth.get_session().get(
            f"{self.auth.get_api_url_base()}{path}", headers=self.headers, params=params
        )
        return rs.json()

    def get_game_schedule(self, filter: str) -> dict:
        return self.get_request(
            "/data/record/game_schedule",
            {
                "filter": filter,
            },
        )

    def get_pitch_data(self, filter: str, limit=300) -> dict:
        return self.get_request(
            "/data/mashup/pitch_data",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_player_master(self, filter: str) -> dict:
        return self.get_request(
            "/data/master/player_master",
            {
                "filter": filter,
            },
        )

    def get_umpire_master(self, filter: str, limit=1000) -> dict:
        return self.get_request(
            "/data/master/umpire_master",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_team_master(self, filter: str) -> dict:
        return self.get_request(
            "/data/master/team_master",
            {
                "filter": filter,
            },
        )

    def get_stadium_master(self, filter: str) -> dict:
        return self.get_request(
            "/data/master/stadium_master",
            {
                "filter": filter,
            },
        )

    def get_movie_angle_upload(self, filter: str, limit: int = 5) -> dict:
        return self.get_request(
            "/movie/angle/upload",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_movie_angle(self, filter: str) -> dict:
        return self.get_request(
            "/movie/angle",
            {
                "filter": filter,
            },
        )

    def get_trackman_record(self, filter: str, limit: int = 300) -> dict:
        return self.get_request(
            "/data/mashup/trackman_record",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_charlyze_record(self, resource_name: str, filter: str) -> dict:
        return self.get_request(
            f"/data/charlyze/record/{resource_name}",
            {
                "filter": filter,
            },
        )

    def get_event_info(self, filter: str, limit: int = 300) -> dict:
        return self.get_request(
            "/data/record/event_info",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_batter_record(self, filter: str, limit: int = 300) -> dict:
        return self.get_request(
            "/data/record/batter_record",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def get_pitcher_record(self, filter: str, limit: int = 300) -> dict:
        return self.get_request(
            "/data/record/pitcher_record",
            {
                "filter": filter,
                "limit": limit,
            },
        )

    def download_movie_angle_upload_movie_file_path(
        self, filter: str, download_dir: str
    ) -> Tuple[dict, list]:
        rs = self.get_movie_angle_upload(filter)
        files = self._download_files(rs.get("results"), "movie_file_path", download_dir)
        return rs, files

    def download_ts_from_m3u8(self, m3u8_url: str, download_dir: str) -> list:
        base_uri, m3u8_basename = os.path.split(m3u8_url)
        local_path = f"{download_dir}/{m3u8_basename}"
        self.download(local_path=local_path, url=m3u8_url)
        playlist = m3u8.load(local_path)
        segments = playlist.segments.by_key(None)
        files = []
        for segment in segments:
            ts_basename = segment.uri
            ts_local_path = f"{download_dir}/{ts_basename}"
            if os.path.exists(ts_local_path):
                self.logger.debug(f"already exists => {ts_local_path}")
            else:
                segment_uri = f"{base_uri}/{ts_basename}"
                self.download(local_path=ts_local_path, url=segment_uri)
            files.append(ts_local_path)
        return files

    def download_pitch_data_playlist_file_path(self, filter: str, download_dir: str):
        rs = self.get_pitch_data(filter)
        files = self._download_files(
            rs.get("results"), "playlist_file_path", download_dir
        )
        return rs, files

    def _download_files(self, results, key: str, download_dir: str):
        files = []
        os.makedirs(download_dir, exist_ok=True)
        for result in results:
            url = result.get(key)
            local_path = f"{download_dir}/{os.path.basename(url)}"
            if os.path.exists(local_path):
                self.logger.debug(f"already exists => {local_path}")
            else:
                self.download(local_path, url)
            files.append(local_path)
        return files

    def download(self, local_path: str, url: str):
        self.logger.debug(f"stream downloading... => {local_path}")
        response = self.auth.get_session().get(
            url,
            stream=True,
        )
        path = Path(local_path)
        with open(path, "wb") as f:
            for chunk in response.iter_content(chunk_size=512 * 1024):
                f.write(chunk)
        size_gb = os.path.getsize(path) / 1024 / 1024 / 1024
        self.logger.debug(f"download done. => {local_path} ({size_gb} GB)")



import requests
import os
from .config import Config
from . import constants


class Auth:
    def __init__(
        self,
        config_path: str = constants.DEFAULT_CONFIG_PATH,
        config_profile: str = constants.DEFAULT_CONFIG_PROFILE,
        endpoint: str = None,
        organization_code: str = None,
        username: str = None,
        password: str = None,
        grant_type: str = None,
    ):
        if username is not None:
            self.config = Config(
                {
                    "endpoint": endpoint,
                    "organization_code": organization_code,
                    "username": username,
                    "password": password,
                    "grant_type": grant_type,
                }
            )
        elif os.environ.get("PITCHBASE_USERNAME") is not None:
            self.config = Config(
                {
                    "endpoint": os.environ.get("PITCHBASE_ENDPOINT"),
                    "organization_code": os.environ.get("PITCHBASE_ORGANIZATION_CODE"),
                    "username": os.environ.get("PITCHBASE_USERNAME"),
                    "password": os.environ.get("PITCHBASE_PASSWORD"),
                    "grant_type": os.environ.get("PITCHBASE_GRANT_TYPE"),
                }
            )
        elif config_path is not None:
            self.config = Config.read(config_path, config_profile)
        else:
            raise Exception("Unable to authenticate")

        self.api_url_base = (
            f"https://{self.config.endpoint}/apiv1/{self.config.organization_code}"
        )
        self.session = requests.Session()

    def get_access_token(self, reset: bool = False) -> str:
        if os.environ.get("PITCHBASE_API_TOKEN") is not None and (not reset):
            return os.environ.get("PITCHBASE_API_TOKEN")

        rs = self.session.post(
            f"{self.api_url_base}/auth",
            data={
                "username": self.config.username,
                "password": self.config.password,
                "grant_type": self.config.grant_type,
            },
            headers={
                "Content-Type": "application/x-www-form-urlencoded",
            },
        )
        return rs.json()["access_token"]

    def get_header(self) -> dict:
        return {
            "X-PitchBase-API-Token": self.get_access_token(),
            "Accept": "application/json",
        }

    def get_api_url_base(self) -> str:
        return self.api_url_base

    def get_session(self) -> requests.Session:
        return self.session



class TestUtil(object):
    def setup_method(self, method):
        pass

    def teardown_method(self, method):
        pass

    def test_pad_zero(self):
        # assert Util.get("2020-03") == 0
        pass



import yaml
from . import constants


class Config:
    def __init__(self, cfg):
        self._cfg = cfg

    def __getattr__(self, k):
        v = self._cfg[k]
        if isinstance(v, dict):
            return Config(v)
        return v

    @staticmethod
    def read(path: str, profile: str):
        profile = profile or constants.DEFAULT_CONFIG_PROFILE
        c = yaml.safe_load(open(path, encoding="UTF-8").read()).get(profile)
        return Config(c)



from setuptools import setup, find_packages
import os

here = os.path.abspath(os.path.dirname(__file__))
install_requires = ["click>=7.0", "requests", "pyyaml", "m3u8"]
extras_require = {"test": ["tox"]}

setup(
    name="pitchbase",
    version="0.13.2",
    description="Command Line utility for pitchbase.",
    long_description=open(os.path.join(here, "README.md")).read(),
    long_description_content_type="text/markdown",
    classifiers=[
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="pitchbase tool aws",
    author="Hiroshi Toyama",
    author_email="toyama0919@gmail.com",
    url="https://github.dena.jp/AI/pitchbase",
    license="MIT",
    packages=find_packages("src", exclude=["tests"]),
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    extras_require=extras_require,
    tests_require=extras_require["test"],
    entry_points={"console_scripts": ["pitchbase=pitchbase.commands:main"]},
)
